jQuery(function ($) {
    'use strict';

    let truevoPaymentProcessStarted = false;
    let truevoPaymentResponseReceived = false;

    // Initial logging
    let initialMessage = wc_truevo_checkout_widget_params.is_inline ? 'Checkout page loaded for ' + wc_truevo_checkout_widget_params.identifier + '.' : 'Truevo pay now page loaded for ' + wc_truevo_checkout_widget_params.identifier + '.';
    window.wc_truevo_payment_request.postMessageToLog(initialMessage);

    // Configuration checks
    var configuration = wc_truevo_checkout_widget_params.configuration;

    if (!configuration) {
        window.wc_truevo_payment_request.postMessageToLog('Error reading configuration for ' + wc_truevo_checkout_widget_params.identifier + '.');
    }

    // Call loading of Truevo payment widget
    if (wc_truevo_checkout_widget_params.is_inline) {
        // Selector string
        const paymentMethodSelector = "form[name='checkout'] input[name='payment_method']";
        const paymentGatewayName = "truevo_gateway";
        let hasLoaded = false;

        // If Truevo payment is selected by default, load the widget...
        let selectedPaymentMethod = $(paymentMethodSelector + ":checked:first");
        if (selectedPaymentMethod && selectedPaymentMethod.val() === paymentGatewayName) {
            loadTruevoWidget();
            hasLoaded = true;
        }

        // ... else detect when it is selected and then load it
        $(document).on("change", paymentMethodSelector, function() {
            if (!hasLoaded && $(this).val() === paymentGatewayName) {
                loadTruevoWidget();
                hasLoaded = true;
            }
        //disable Place Order button with Truevo Payment method until relevant input fields filled
        const payByCardRdBtn = document.getElementById('payment_method_truevo_gateway'); 
        const payButton = document.getElementById('place_order');
            if(payByCardRdBtn.checked == false){
                    payButton.removeAttribute('disabled');
                    $("#place_order").css("cursor", "default");
                }
                else if(payByCardRdBtn.checked == true) {
                    payButton.setAttribute('disabled', true);
                    $("#place_order").css("cursor", "not-allowed");
                }
        });
    } else {
        // Immediately load widget on "Redirect" flow
        loadTruevoWidget();
    }
    //widget reload counter
    let reloadCount = 0;

    // Widget setup and event listeners function
    function loadTruevoWidget() {
        try {
            window.wc_truevo_payment_request.postMessageToLog('Initializing Truevo Checkout for ' + wc_truevo_checkout_widget_params.identifier + '.');

            TruevoCheckout.initialize(configuration);
        } catch (error) {
            window.wc_truevo_payment_request.postMessageToLog('Error initializing Truevo Checkout for ' + wc_truevo_checkout_widget_params.identifier + '.', {
                exception: error
            });
        }

        try {
            window.wc_truevo_payment_request.postMessageToLog('Mounting Truevo Checkout for ' + wc_truevo_checkout_widget_params.identifier + '.');

            const truevoCheckout = TruevoCheckout.mount('#truevo_form_container');

            truevoCheckout.addEventListener('configurationReceived', res => {
                let message = 'Truevo Checkout Event: configurationReceived for ' + wc_truevo_checkout_widget_params.identifier + '.';
                window.wc_truevo_payment_request.postMessageToLog(message, res.detail);
            });

            truevoCheckout.addEventListener('payButtonClicked', res => {
                $("#truevo_cancel_button").removeAttr('href').css("cursor", "not-allowed");
                $(".modal-truevo-payment-toggle").addClass('active');
                $("a.truevo-payment-close").addClass('active');

                let message = 'Truevo Checkout Event: payButtonClicked for ' + wc_truevo_checkout_widget_params.identifier + '.';
                window.wc_truevo_payment_request.postMessageToLog(message, res.detail);
                truevoPaymentProcessStarted = true;
            });

            truevoCheckout.addEventListener('loadCompleted', res => {
                console.log('======== Payment Widget Loaded ==========');

                const payButton = document.getElementById('place_order');
                    if (payButton) {
                        payButton.setAttribute('disabled', true);
                        $("#place_order").css("cursor", "not-allowed");
                    }
                
                let iframe = $("#truevo_form_iframe");
                    if(iframe.height() === null && reloadCount === 0) {
                        reloadCount++;
                        loadTruevoWidget();
                    }

                let message = 'Truevo Checkout Event: loadCompleted for ' + wc_truevo_checkout_widget_params.identifier + '.';
                let details = {
                    'src': iframe.attr('src'),
                    'dimensions': iframe.width() + 'x' + iframe.height(),
                    'response': res.detail
                };
                window.wc_truevo_payment_request.postMessageToLog(message, details);
            });

            truevoCheckout.addEventListener('loadFailed', res => {
                console.log('======== Payment Widget Failed To Load ==========');

                let message = 'Truevo Checkout Event: loadFailed for ' + wc_truevo_checkout_widget_params.identifier + '.';
                window.wc_truevo_payment_request.postMessageToLog(message, res.detail);
            });

               /* Inline PayButton */
            truevoCheckout.addEventListener('enablePayButton', res => {
                /* Enable pay button event */
                console.log('========Enable Pay Button==========');
                const payButton = document.getElementById('place_order');
                if (payButton) {
                payButton.removeAttribute('disabled');
                $("#place_order").css("cursor" , "default");
                }
            });
        
            truevoCheckout.addEventListener('disablePayButton', res => {
                /* Disable pay button event */
                console.log('========Disable Pay Button==========');
                const payButton = document.getElementById('place_order');
                if (payButton) {
                payButton.setAttribute('disabled', true);
                $("#place_order").css("cursor", "not-allowed");
                }
            });

            truevoCheckout.addEventListener('transactionCompleted', res => {
                updateOrderWithTruevoDetails(res.detail);
            });

            truevoCheckout.addEventListener('transactionDeclined', res => {
                updateOrderWithTruevoDetails(res.detail);
            });

            truevoCheckout.addEventListener('transactionFailed', res => {
                updateOrderWithTruevoDetails(res.detail);
            });

            if (wc_truevo_checkout_widget_params.is_inline) {
                truevoCheckout.addEventListener('applePayButtonClicked', res => {
                    truevoAddBlockingOverlayOnThirdPartyPayment(res, 'applePayButtonClicked');

                    TruevoCheckout.triggerApplePay({
                        totalAmount: configuration.transaction.amount.totalAmount,
                        currencyCode: configuration.transaction.amount.currencyAlphaCode
                    });
                });

                truevoCheckout.addEventListener('applePayOnPaymentAuthorizedSuccess', res => {
                    truevoAutoSubmitPlaceOrder(res, 'applePayOnPaymentAuthorizedSuccess ');
                });

                truevoCheckout.addEventListener('applePayOnPaymentAuthorizedError', res => {
                    truevoRemoveBlockingOverlay(res, 'applePayOnPaymentAuthorizedError ');
                });

                truevoCheckout.addEventListener('googlePayButtonClicked', res => {
                    truevoAddBlockingOverlayOnThirdPartyPayment(res, 'googlePayButtonClicked');

                    TruevoCheckout.triggerGooglePay({
                        totalAmount: configuration.transaction.amount.totalAmount,
                        currencyCode: configuration.transaction.amount.currencyAlphaCode
                    });
                });

                truevoCheckout.addEventListener('googlePayConfirmButtonClicked', res => {
                    truevoAutoSubmitPlaceOrder(res, 'googlePayConfirmButtonClicked');
                });

                truevoCheckout.addEventListener('googlePayPaymentAuthorizedError', res => {
                    truevoRemoveBlockingOverlay(res, 'googlePayPaymentAuthorizedError');
                });
            }
        } catch (error) {
            window.wc_truevo_payment_request.postMessageToLog('Error Mounting Truevo Checkout for ' + wc_truevo_checkout_widget_params.identifier + '.', {
                exception: error
            });
        }
    }

    // Cancel order (only works for "Redirect" flow)
    $("#truevo_cancel_button").on('click', function() {
        if ($(this).attr('href') !== undefined) {
            let message = 'Truevo Checkout cancel button clicked for ' + wc_truevo_checkout_widget_params.identifier + '.';
            window.wc_truevo_payment_request.postMessageToLog(message);
        }
    });

    // Serialize and submit form
    function updateOrderWithTruevoDetails(response) {
        $("#truevo-order-flag").val(response.status);
        $("#truevo-order-values").val(JSON.stringify(response.transactionInfo));
        $("#truevo-order-values-all").val(JSON.stringify(response));
        truevoPaymentResponseReceived = true;

        let form = $('form[name="transaction_complete"]');

        if (wc_truevo_checkout_widget_params.is_inline) {
                let url = form.attr('action');
                let data = form.serialize();
    
               
        setTimeout(function(){

            $.ajax({
                url: url,
                type: 'POST',
                data: data,
                success: function(result) {
                    if (result && result.redirect_url) {
                        window.location.href = result.redirect_url;
                    } else {
                        window.location.reload();
                    }
                }
            });


        }, 5000);

        } else {
            setTimeout(function () {
                form.submit();
            }, 5000);
        }
    }

    window.addEventListener('pagehide', (event) => {

        if (truevoPaymentProcessStarted && !truevoPaymentResponseReceived) {
            let message = "Payment Interruption Log: Payment process started but page closed/disrupted before response is received): " + wc_truevo_checkout_widget_params.identifier  + '. Amount ' + wc_truevo_checkout_widget_params.configuration.transaction.amount.totalAmount + '.';
            let details = '';
            window.wc_truevo_payment_request.postMessageToLog(message, details, true); 
            event.preventDefault();
          }
      });
    
    function truevoAddBlockingOverlayOnThirdPartyPayment(response, action) {
        let message = 'Truevo Checkout Event: ' + action + ' for ' + wc_truevo_checkout_widget_params.identifier + '.';
        window.wc_truevo_payment_request.postMessageToLog(message, response.detail);

        $('form.woocommerce-checkout').block({
            message: null,
            overlayCSS: {
                background: "#fff",
                opacity: .6
            }
        });
    }

    function truevoRemoveBlockingOverlay(response, action) {
        let message = 'Truevo Checkout Event: ' + action + ' for ' + wc_truevo_checkout_widget_params.identifier + '.';
        window.wc_truevo_payment_request.postMessageToLog(message, response.detail);

        $('form.woocommerce-checkout').unblock();
    }

    function truevoAutoSubmitPlaceOrder(response, action) {
        let message = 'Truevo Checkout Event: ' + action + ' for ' + wc_truevo_checkout_widget_params.identifier + '.';
        window.wc_truevo_payment_request.postMessageToLog(message, response.detail);

        $('form.checkout').submit();
    }
});